/*
* Line follower
* Clit Alina 335CA
* PM - proiect
*/

#define F_CPU 16000000


#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define MAX_SPEED	150
#define LEFT		1
#define RIGHT		2
#define DELAY 	    50

#define DEFAULT_PANIC		RIGHT

/* initializations */
void init()
{

	/* port A input*/
	DDRA = 0x00;
	PORTA = 0xFF;

	/* port B output*/
	DDRB = 0xFF;
	PORTB = 0x00;

	/* port D output*/
	DDRD = 0xFF;
	PORTD = 0x00;

	/* PWM */
	DDRD |= (1 << PD4) | (1 << PD5);

	/*timer 1*/
	TCCR1A |= (1 << COM1A1) | (1 << COM1B1) | (1 << WGM10);
	TCCR1B |= (1 << WGM12) | (1 << CS12);

	PORTB |= (1 << PB0);
	PORTB &= ~(1 << PB1);
	PORTB |= (1 << PB2);
	PORTB &= ~(1 << PB3);

}

/* set motor power */
void move(int speedLeft , int speedRight)
{
	/* left motor power */
	OCR1A = speedLeft;

	/* right motor power */
	OCR1B = speedRight;
}

int main()
{
	int panic = DEFAULT_PANIC;
	int left, right, dirL, dirR;
	int err = 0;

	/* init */
	init();

	while (1)
	{

		left = 0;
		right = 0;
		dirL = 0;
		dirR = 0;

		/* left sensors */
		if (PINA & (1 << PA0)) {
			left++;
			dirL = 4;
		}
		if (PINA & (1 << PA1)) {
			left++;
			dirL = 3;
		}
		if (PINA & (1 << PA2)) {
			left++;
			dirL = 2;
		}
		if (PINA & (1 << PA3)) {
			left++;
			dirL = 1;
		}

		/* right sensors */
		if (PINA & (1 << PA7)) {
			right++;
			dirR = 4;
		}
		if (PINA & (1 << PA6)) {
			right++;
			dirR = 3;
		}
		if (PINA & (1 << PA5)) {
			right++;
			dirR = 2;
		}
		if (PINA & (1 << PA4)) {
			right++;
			dirR = 1;
		}

		/*blake line in the middle*/
		if (dirR != 0 && dirL != 0) {
			if (left == right)
				move(MAX_SPEED, MAX_SPEED);
			if (left > right) {
				if (left == 4) {
					move(0, MAX_SPEED  * 1.1);
				}
				else {
					if ((left - right) == 1)
						move(MAX_SPEED * 0.9, MAX_SPEED);
					if ((left - right) == 2)
						move(MAX_SPEED * 0.8, MAX_SPEED);
				}
			}
			else {
				if (right == 4) {
					move(MAX_SPEED * 1.1, 0);
				}
				else {
					if ((right - left) == 1)
						move(MAX_SPEED, MAX_SPEED * 0.9);
					if ((right - left) == 2)
						move(MAX_SPEED, MAX_SPEED * 0.8);
				}

			}
			err = 0;
		}

		/* too left */
		if (dirL != 0 && dirR == 0) {
			if (dirL == 3 || dirL == 4)
				move(0, MAX_SPEED);
			if (dirL == 2)
				move(MAX_SPEED * 0.4, MAX_SPEED);
			if (dirL == 1) {
				if (left == 4)
					move(MAX_SPEED * 0, MAX_SPEED * 1.2);
				else if (left == 3)
					move(MAX_SPEED * 0.5, MAX_SPEED);
				else
					move(MAX_SPEED * 0.7, MAX_SPEED);
			}
			panic = RIGHT;
			err = 0;
		}

		/* too right */
		if (dirR != 0 && dirL == 0) {
			if (dirR == 3 || dirR == 4)
				move(MAX_SPEED, 0);
			if (dirR == 2)
				move(MAX_SPEED, MAX_SPEED * 0.4);
			if (dirR == 1) {
				if (right == 4)
					move(MAX_SPEED * 1.2, MAX_SPEED * 0);
				else if ( right == 3)
					move(MAX_SPEED, MAX_SPEED * 0.5);
				else
					move(MAX_SPEED, MAX_SPEED * 0.7);
			}
			panic = LEFT;
			err = 0;
		}

		/* panic */
		if (dirL == 0 && 0 == dirR) {
			if (err < 5000) {
				if (panic == LEFT)
					move(0, MAX_SPEED * 0.8);
				else
					move(MAX_SPEED * 0.8, 0);
				err++;
			} else
				move(0, 0);
		}

		_delay_ms(DELAY);
	}
	
	return 0;
}